@@ -1,7 +1,5 @@
 # Ultralytics 🚀 AGPL-3.0 License - https://ultralytics.com/license
 
-import requests
-
 from ultralytics.data.utils import HUBDatasetStats
 from ultralytics.hub.auth import Auth
 from ultralytics.hub.session import HUBTrainingSession
@@ -75,6 +73,8 @@ def logout():
 
 def reset_model(model_id: str = ""):
     """Reset a trained model to an untrained state."""
+    import requests  # scoped as slow import
+
     r = requests.post(f"{HUB_API_ROOT}/model-reset", json={"modelId": model_id}, headers={"x-api-key": Auth().api_key})
     if r.status_code == 200:
         LOGGER.info(f"{PREFIX}Model reset successfully")
@@ -105,6 +105,8 @@ def export_model(model_id: str = "", format: str = "torchscript"):
         >>> from ultralytics import hub
         >>> hub.export_model(model_id="your_model_id", format="torchscript")
     """
+    import requests  # scoped as slow import
+
     assert format in export_fmts_hub(), f"Unsupported export format '{format}', valid formats are {export_fmts_hub()}"
     r = requests.post(
         f"{HUB_API_ROOT}/v1/models/{model_id}/export", json={"format": format}, headers={"x-api-key": Auth().api_key}
@@ -132,6 +134,8 @@ def get_export(model_id: str = "", format: str = "torchscript"):
         >>> from ultralytics import hub
         >>> result = hub.get_export(model_id="your_model_id", format="torchscript")
     """
+    import requests  # scoped as slow import
+
     assert format in export_fmts_hub(), f"Unsupported export format '{format}', valid formats are {export_fmts_hub()}"
     r = requests.post(
         f"{HUB_API_ROOT}/get-export",
