@@ -4,7 +4,7 @@ import copy
 from typing import Optional
 
 import torch
-from torch import Tensor, nn
+from torch import nn
 
 from .blocks import RoPEAttention
 
@@ -103,7 +103,7 @@ class MemoryAttentionLayer(nn.Module):
         self.pos_enc_at_cross_attn_queries = pos_enc_at_cross_attn_queries
         self.pos_enc_at_cross_attn_keys = pos_enc_at_cross_attn_keys
 
-    def _forward_sa(self, tgt: Tensor, query_pos: Optional[Tensor]) -> Tensor:
+    def _forward_sa(self, tgt: torch.Tensor, query_pos: Optional[torch.Tensor]) -> torch.Tensor:
         """Perform self-attention on input tensor using positional encoding and RoPE attention mechanism."""
         tgt2 = self.norm1(tgt)
         q = k = tgt2 + query_pos if self.pos_enc_at_attn else tgt2
@@ -113,12 +113,12 @@ class MemoryAttentionLayer(nn.Module):
 
     def _forward_ca(
         self,
-        tgt: Tensor,
-        memory: Tensor,
-        query_pos: Optional[Tensor],
-        pos: Optional[Tensor],
+        tgt: torch.Tensor,
+        memory: torch.Tensor,
+        query_pos: Optional[torch.Tensor],
+        pos: Optional[torch.Tensor],
         num_k_exclude_rope: int = 0,
-    ) -> Tensor:
+    ) -> torch.Tensor:
         """Perform cross-attention between target and memory tensors using RoPEAttention mechanism."""
         kwds = {}
         if num_k_exclude_rope > 0:
@@ -138,20 +138,20 @@ class MemoryAttentionLayer(nn.Module):
 
     def forward(
         self,
-        tgt: Tensor,
-        memory: Tensor,
-        pos: Optional[Tensor] = None,
-        query_pos: Optional[Tensor] = None,
+        tgt: torch.Tensor,
+        memory: torch.Tensor,
+        pos: Optional[torch.Tensor] = None,
+        query_pos: Optional[torch.Tensor] = None,
         num_k_exclude_rope: int = 0,
     ) -> torch.Tensor:
         """
         Process input tensors through self-attention, cross-attention, and feedforward network layers.
 
         Args:
-            tgt (Tensor): Target tensor for self-attention with shape (N, L, D).
-            memory (Tensor): Memory tensor for cross-attention with shape (N, S, D).
-            pos (Optional[Tensor]): Positional encoding for memory tensor.
-            query_pos (Optional[Tensor]): Positional encoding for target tensor.
+            tgt (torch.Tensor): Target tensor for self-attention with shape (N, L, D).
+            memory (torch.Tensor): Memory tensor for cross-attention with shape (N, S, D).
+            pos (Optional[torch.Tensor]): Positional encoding for memory tensor.
+            query_pos (Optional[torch.Tensor]): Positional encoding for target tensor.
             num_k_exclude_rope (int): Number of keys to exclude from rotary position embedding.
 
         Returns:
@@ -242,8 +242,8 @@ class MemoryAttention(nn.Module):
         self,
         curr: torch.Tensor,  # self-attention inputs
         memory: torch.Tensor,  # cross-attention inputs
-        curr_pos: Optional[Tensor] = None,  # pos_enc for self-attention inputs
-        memory_pos: Optional[Tensor] = None,  # pos_enc for cross-attention inputs
+        curr_pos: Optional[torch.Tensor] = None,  # pos_enc for self-attention inputs
+        memory_pos: Optional[torch.Tensor] = None,  # pos_enc for cross-attention inputs
         num_obj_ptr_tokens: int = 0,  # number of object pointer *tokens*
     ) -> torch.Tensor:
         """
@@ -252,8 +252,8 @@ class MemoryAttention(nn.Module):
         Args:
             curr (torch.Tensor): Self-attention input tensor, representing the current state.
             memory (torch.Tensor): Cross-attention input tensor, representing memory information.
-            curr_pos (Optional[Tensor]): Positional encoding for self-attention inputs.
-            memory_pos (Optional[Tensor]): Positional encoding for cross-attention inputs.
+            curr_pos (Optional[torch.Tensor]): Positional encoding for self-attention inputs.
+            memory_pos (Optional[torch.Tensor]): Positional encoding for cross-attention inputs.
             num_obj_ptr_tokens (int): Number of object pointer tokens to exclude from rotary position embedding.
 
         Returns:
