@@ -1,6 +1,7 @@
 # Ultralytics 🚀 AGPL-3.0 License - https://ultralytics.com/license
 
 from copy import deepcopy
+from pathlib import Path
 from typing import Any, Dict, Optional, Union
 
 import torch
@@ -180,9 +181,9 @@ class YOLOEDetectValidator(DetectionValidator):
         else:
             if refer_data is not None:
                 assert load_vp, "Refer data is only used for visual prompt validation."
-            self.device = select_device(self.args.device)
+            self.device = select_device(self.args.device, verbose=False)
 
-            if isinstance(model, str):
+            if isinstance(model, (str, Path)):
                 from ultralytics.nn.tasks import attempt_load_weights
 
                 model = attempt_load_weights(model, device=self.device, inplace=True)
