@@ -1,6 +1,7 @@
 # Ultralytics 🚀 AGPL-3.0 License - https://ultralytics.com/license
 
 from collections import defaultdict
+from typing import Any
 
 from ultralytics.solutions.solutions import BaseSolution, SolutionAnnotator, SolutionResults
 
@@ -30,7 +31,7 @@ class AIGym(BaseSolution):
         >>> cv2.waitKey(0)
     """
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs: Any) -> None:
         """
         Initialize AIGym for workout monitoring using pose estimation and predefined angles.
 
@@ -47,7 +48,7 @@ class AIGym(BaseSolution):
         self.down_angle = float(self.CFG["down_angle"])  # Pose down predefined angle to consider down pose
         self.kpts = self.CFG["kpts"]  # User selected kpts of workouts storage for further usage
 
-    def process(self, im0):
+    def process(self, im0) -> SolutionResults:
         """
         Monitor workouts using Ultralytics YOLO Pose Model.
 
@@ -74,15 +75,12 @@ class AIGym(BaseSolution):
         annotator = SolutionAnnotator(im0, line_width=self.line_width)  # Initialize annotator
 
         self.extract_tracks(im0)  # Extract tracks (bounding boxes, classes, and masks)
-        tracks = self.tracks[0]
 
-        if tracks.boxes.id is not None:
-            track_ids = tracks.boxes.id.cpu().tolist()
-            kpt_data = tracks.keypoints.data.cpu()  # Avoid repeated .cpu() calls
+        if len(self.boxes):
+            kpt_data = self.tracks.keypoints.data
 
             for i, k in enumerate(kpt_data):
-                track_id = int(track_ids[i])  # get track id
-                state = self.states[track_id]  # get state details
+                state = self.states[self.track_ids[i]]  # get state details
                 # Get keypoints and estimate the angle
                 state["angle"] = annotator.estimate_pose_angle(*[k[int(idx)] for idx in self.kpts])
                 annotator.draw_specific_kpts(k, self.kpts, radius=self.line_width * 3)
