@@ -115,15 +115,17 @@ class RegionCounter(BaseSolution):
 
         # Display region counts
         for region in self.counting_regions:
-            x1, y1, x2, y2 = map(int, region["polygon"].bounds)
-            pts = [(x1, y1), (x2, y1), (x2, y2), (x1, y2)]
+            poly = region["polygon"]
+            pts = list(map(tuple, np.array(poly.exterior.coords, dtype=np.int32)))
+            (x1, y1), (x2, y2) = [(int(poly.centroid.x), int(poly.centroid.y))] * 2
             annotator.draw_region(pts, region["region_color"], self.line_width * 2)
-            annotator.text_label(
+            annotator.adaptive_label(
                 [x1, y1, x2, y2],
                 label=str(region["counts"]),
                 color=region["region_color"],
                 txt_color=region["text_color"],
                 margin=self.line_width * 4,
+                shape="rect",
             )
             region["counts"] = 0  # Reset for next frame
         plot_im = annotator.result()
