@@ -11,7 +11,7 @@ class TritonRemoteModel:
     Client for interacting with a remote Triton Inference Server model.
 
     This class provides a convenient interface for sending inference requests to a Triton Inference Server
-    and processing the responses.
+    and processing the responses. Supports both HTTP and gRPC communication protocols.
 
     Attributes:
         endpoint (str): The name of the model on the Triton server.
@@ -31,6 +31,7 @@ class TritonRemoteModel:
     Examples:
         Initialize a Triton client with HTTP
         >>> model = TritonRemoteModel(url="localhost:8000", endpoint="yolov8", scheme="http")
+
         Make inference with numpy arrays
         >>> outputs = model(np.random.rand(1, 3, 640, 640).astype(np.float32))
     """
@@ -44,8 +45,8 @@ class TritonRemoteModel:
 
         Args:
             url (str): The URL of the Triton server.
-            endpoint (str): The name of the model on the Triton server.
-            scheme (str): The communication scheme ('http' or 'grpc').
+            endpoint (str, optional): The name of the model on the Triton server.
+            scheme (str, optional): The communication scheme ('http' or 'grpc').
 
         Examples:
             >>> model = TritonRemoteModel(url="localhost:8000", endpoint="yolov8", scheme="http")
@@ -53,7 +54,7 @@ class TritonRemoteModel:
         """
         if not endpoint and not scheme:  # Parse all args from URL string
             splits = urlsplit(url)
-            endpoint = splits.path.strip("/").split("/")[0]
+            endpoint = splits.path.strip("/").split("/", 1)[0]
             scheme = splits.scheme
             url = splits.netloc
 
@@ -87,7 +88,7 @@ class TritonRemoteModel:
 
     def __call__(self, *inputs: np.ndarray) -> List[np.ndarray]:
         """
-        Call the model with the given inputs.
+        Call the model with the given inputs and return inference results.
 
         Args:
             *inputs (np.ndarray): Input data to the model. Each array should match the expected shape and type
